
from .base_calibrations import BaseCalibration


class IbeltCalibation(BaseCalibration):
    """Calibrations performed inside the utube
    """

    _ALIAS = "ibelt_calibrations"

    z_cm: float
    plug: bool



from typing import Literal


SOURCE = Literal['ambe',
 'ar-37',
 'kr-83m',
 'rn-220',
 'led',
 'pulser',
 'noise',
 'none',
 'th-228',
 'th-232',
]

DIFFUSED_SOURCE = Literal['rn-220','kr-83m','ar-37']

DETECTOR = Literal['tpc','neutron_veto','muon_veto']



import pydantic
import datetime
from typing import Literal, List
from .base_report import BaseOperationsReport


class WarmPmt(pydantic.BaseModel):
    pmt: int = pydantic.Field(lt=494, ge=0)
    avg_rate_kbps: float


class WarmspotReport(BaseOperationsReport):
    """Warmspot report
    """

    _ALIAS = "warmspot_reports"
    
    pmts: List[WarmPmt]





import rframe
import datetime
from typing import List
from pydantic import Field, BaseModel


from .base_pmt_data import BasePmtData
from ..constants import DETECTOR


class VoltageChange(BasePmtData):
    
    detector: DETECTOR = rframe.Index()
    pmt: int = rframe.Index(ge=0)
    time: datetime.datetime = rframe.Index()

    old_voltage: float
    new_voltage: float
    operator: str = Field(max_length=60)
    comments: str


from typing import Any, Dict

import pytz
import rframe

from rframe import RemoteFrame

from .schemas import XeDoc
from ._settings import settings


class SchemaFrames:
    _cache: Dict[str,RemoteFrame] = {}
    db: Any
    
    def __init__(self, db=None):
        self.db = db

    @classmethod
    def from_mongodb(cls, url="localhost", db="cmt2", **kwargs):
        import pymongo

        db = pymongo.MongoClient(url, **kwargs)[db]
        return cls(db)

    @property
    def schemas(self):
        return dict(XeDoc._XEDOCS)

    @property
    def schema_names(self):
        return list(self.schemas)

    def get_rf(self, name):
        if name not in self._cache:
            import xedocs

            schema = xedocs.find_schema(name)
            self._cache[name] = schema.rframe()
            
        return self._cache[name]
        
    def __getitem__(self, key):
        if isinstance(key, tuple) and key[0] in self.schemas:
            return self.get_rf(key[0])[key[1:]]

        if key in self.schemas:
            return self.get_rf(key)
        raise KeyError(key)

    def __dir__(self):
        return super().__dir__() + list(self.schemas)

    def __getattr__(self, name):
        if name != "schemas" and name in self.schemas:
            return self.get_rf(name)
        return super().__getattribute__(name)

    def sel(self, schema_name, *args, **kwargs):
        return self.get_rf(schema_name).sel(*args, **kwargs)

    def set(self, schema_name, *args, **kwargs):
        return self.get_rf(schema_name).set(*args, **kwargs)

    def insert(self, schema_name, records):
        return self.get_rf(schema_name).insert(records=records)


frames = SchemaFrames()



from ast import Import
import pandas as pd

try:
    import utilix
    uconfig = utilix.uconfig
    from utilix import xent_collection

except ImportError:
    uconfig = None
    def xent_collection(**kwargs):
        raise RuntimeError('utilix not configured')

from pydantic import BaseSettings

from .clock import SimpleClock

class Settings(BaseSettings):
    class Config:
        env_prefix = 'XEDOCS_'

    DEFAULT_DATABASE: str = "xedocs"

    API_URL: str = 'https://api.xedocs.yossisprojects.com'
    API_VERSION: str = 'v1'
    API_AUDIENCE: str = 'https://api.cmt.xenonnt.org'
    API_WRITE: bool = False
    API_TOKEN: str = None
    API_USERNAME: str = None
    API_PASSWORD: str = None

    clock = SimpleClock()

    datasources = {}

    @property
    def api_token(self):
        import xedocs

        if self.API_TOKEN is None:
            readonly = not self.API_WRITE
            token = xedocs.api.api_token(self.API_USERNAME,
                                     self.API_PASSWORD,
                                     readonly)
            self.API_TOKEN = token

        return self.API_TOKEN

    def api_client(self, schema):
        import xedocs

        url = '/'.join([self.API_URL.rstrip('/'), self.API_VERSION, schema._ALIAS ])

        return xedocs.api.api_client(url, self.api_token)

    def default_datasource(self, schema):

        if uconfig is not None:
            database = schema.default_database_name()
            if database is None:
                database = self.DEFAULT_DATABASE
            collection = schema.default_collection_name()

            return xent_collection(collection=collection,
                                   database=database)
        return self.api_client(schema)

    def get_datasource_for(self, schema):
        if schema._ALIAS in self.datasources:
            return self.datasources[schema._ALIAS]
        
        return self.default_datasource(schema)

    def run_doc(self, run_id, fields=("start", "end")):
        if uconfig is None:
            raise KeyError(f"Rundb not configured.")
        
        rundb = xent_collection()

        if isinstance(run_id, str):
            run_id = int(run_id)

        query = {"number": run_id}

        doc = rundb.find_one(query, projection={f: 1 for f in fields})
        if not doc:
            raise KeyError(f"Run {run_id} not found.")

        return doc

    def run_id_to_time(self, run_id):
        doc = self.run_doc(run_id)

        # use center time of run
        return doc["start"] + (doc["end"] - doc["start"]) / 2

    def run_id_to_interval(self, run_id):
        doc = self.run_doc(run_id)
        return doc["start"] , doc["end"]

    def extract_time(self, kwargs):
        if "time" in kwargs:
            time = kwargs.pop("time")
        elif "run_id" in kwargs:
            time = self.run_id_to_time(kwargs.pop("run_id"))
        else:
            return None
        return pd.to_datetime(time)


settings = Settings()




from typing import Literal

import rframe

from .base_calibrations import BaseCalibration


class UtubeCalibration(BaseCalibration):
    """Calibrations performed inside the utube
    """

    _ALIAS = "utube_calibrations"

    tube: Literal['top','bottom'] = rframe.Index()
    direction: Literal['cw','ccw'] = rframe.Index()

    depth_cm: float




import datetime
import pydantic
import rframe
from pydantic import validator
from typing import Literal

from ..base_schemas import XeDoc
from ..._settings import settings


class BasePmtData(XeDoc):
    _ALIAS = ""
    _CATEGORY = "pmt_data"

    @classmethod
    def default_database_name(cls):
        return 'xepmts'



"""Main module."""

import pandas as pd

from collections import defaultdict
from typing import ClassVar, Union


from ._settings import settings
from .schemas import XeDoc


def find(schema, datasource=None, **labels):
    if isinstance(schema, str):
        schema = find_schema(schema)
    if not issubclass(schema, XeDoc):
        raise TypeError(
            "Schema must be a subclass of XeDoc" "or the name of a known schema."
        )
    """Find a document in a datasource."""
    # labels, extra = schema.extract_labels(**kwargs)
    return schema.find(datasource, **labels)


def find_df(schema, datasource=None, **labels):
    if isinstance(schema, str):
        schema = find_schema(schema)
    if not issubclass(schema, XeDoc):
        raise TypeError(
            "Schema must be a subclass of XeDoc or the name of a known schema."
        )
    """Find a document in a datasource."""
    return schema.find_df(datasource, **labels)

def find_one(schema, datasource=None, **kwargs):
    if isinstance(schema, str):
        schema = find_schema(schema)
    if not issubclass(schema, XeDoc):
        raise TypeError(
            "Schema must be a subclass of XeDoc or the name of a known schema."
        )
    """Find a document in a datasource."""
    labels, extra = schema.extract_labels(**kwargs)
    return schema.find_one(datasource, **labels)

def insert_docs(schema: str, docs: Union[list,dict,pd.DataFrame], datasource=None):
    if isinstance(docs, pd.DataFrame):
        docs = docs.reset_index().to_dict(orient='records')
    if not isinstance(docs, list):
        docs = [docs]
    if isinstance(schema, str):
        schema = find_schema(schema)

    inserted = []
    for data in docs:
        doc = schema(**data)
        doc.save(datasource)
        inserted.append(doc)

    return inserted

def list_schemas():
    return list(XeDoc._XEDOCS)

def all_schemas():
    return dict(XeDoc._XEDOCS)

def schemas_by_category():
    d = defaultdict(dict)
    for name, schema in all_schemas().items():
        d[schema._CATEGORY][name] = schema 
    return d

def find_schema(name):
    schema = XeDoc._XEDOCS.get(name, None)
    if schema is not None:
        return schema
    for schema in XeDoc._XEDOCS.values():
        if schema.__name__ == name:
            return schema
    raise KeyError(f"Correction with name {name} not found.")


def help(schema):
    if isinstance(schema, str):
        schema = find_schema(schema)
    return schema.help()

def get_api_client(schema):
    if isinstance(schema, str):
        schema = find_schema(schema)
    return settings.api_client(schema)

try:
    '''Attempt to register URConfig protocol if straxen
    is installed.
    '''
    from straxen import URLConfig

    @URLConfig.register("xedocs")
    def xedocs_protocol(name, version="ONLINE", sort=None, attr=None, **kwargs):
        """URLConfig protocol for fetching values from
        correction documents.
        """
        docs = find(name, version=version, **kwargs)

        if not docs:
            raise KeyError(f"No matching documents found for {name}.")

        if isinstance(sort, str):
            docs = sorted(docs, key=lambda x: getattr(x, sort))
        elif sort:
            docs = sorted(docs)

        if attr is not None:
            docs = [getattr(d, attr) for d in docs]

        if len(docs) == 1:
            return docs[0]

        return docs

except ImportError:
    pass



from .base_schemas import VersionedXeDoc, XeDoc
from .bodega import Bodega
from .fax import FaxConfig
from .corrections import *
from .calibrations import *
from .operations_reports import *
from .analysis import *


from .base_corrections import *
from .base_references import *
from .electron_drift import *
from .electron_lifetime import *
from .extraction_efficiency import *
from .field_distortion import *
from .field_simulation import *
from .hit_thresholds import *
from .lone_hits import *
from .pmt_gains import *
from .position_reconstruction import *
from .s1_aft_xyz import *
from .s1_xyz import *
from .s2_xy import *
from .se_gains import *



"""
# Field simulation maps
Using finite element method (FEM) or boundary element method (BEM) analyses it is possible to simulate the electric field inside the TPC and simulate the impact that the field inhomogeneity has on the signal production. In case of freed electrons, some of their observed properties depends on the electric field along their full path (also called "path-weighted"): this is the case for time and position spread or for the drift speed. This can be evaluated by propagating electrons inside the TPC. The following maps have been evaluated so far:

 * electrostatic field map inside the cryostat (not only in the active volume);
 * survival probability within the active volume, meaning the probability of reaching the liquid-gas interface for an electron freed at a given position:
 * path-weighted drift speed;
 * path-weighted diffusion constants (ayimuthal, radial and along z).

In addition, notebooks on the field distortion coming from COMSOL simulations are included, as these are effectively simulation maps mapping the initial and final position.

See [overview on the wiki](https://xe1t-wiki.lngs.infn.it/doku.php?id=xenon:xenonnt:ftoschi:drift_diffusion_tpc_map_low_field).

"""



import rframe

from .base_schemas import VersionedXeDoc


class GlobalVersion(VersionedXeDoc):
    _ALIAS = "global_versions"

    strax_version: str = rframe.Index()
    straxen_version: str = rframe.Index()

    @classmethod
    def get_global_config(cls, version, datasource=None, names=None, extra_labels=None):
        """Build a context config from the given global version."""
        import straxen

        if extra_labels is None:
            extra_labels = dict(run_id="plugin.run_id")
        refs = cls.find(datasource, version=version, alias=names)
        config = {}
        for ref in refs:
            url = ref.url_config
            if extra_labels is not None:
                url = straxen.URLConfig.format_url_kwargs(url, **extra_labels)
            config[ref.alias] = url
        return config



"""
# Field distortion correction
The detector is designed to have uniform electric field, however, it's never perfectly uniform, especially near the TPC edges. If PTFE panels gets charged up, this is even more serious. In order to make analysis much easier, we can correct the field distortion effect. We can do two different approach, the performance is evaluated by studying the uniformity of reconstruct events (Kr83m, Rn220 etc):

 * Simulation driven. It's performance in XENON100 detector was pretty good, but not in XENON1T.
 * Purely data-driven approach. By definition its performance on data is good.

See [description in the Team C overview page](https://xe1t-wiki.lngs.infn.it/doku.php?id=xenon:xenonnt:analysis:reconstruction_team#correction_against_field_distortion)

"""

from .base_references import BaseMap


class FdcMap(BaseMap):
    _ALIAS = "fdc_maps"
    fmt = "json.gz"



import rframe
import datetime

from .base_schemas import VersionedXeDoc


class Bodega(VersionedXeDoc):
    """Detector parameters
    A collection of non-time dependent detector
    values.
    """

    _ALIAS = "bodega"

    field: str = rframe.Index(max_length=50)

    value: float
    uncertainty: float
    definition: str
    reference: str = ''
    date: datetime.datetime




import datetime
import pydantic
import rframe
try:
    import utilix
    uconfig = utilix.uconfig
    from utilix import xent_collection

except ImportError:
    uconfig = None
    def xent_collection(**kwargs):
        raise RuntimeError('utilix not configured')

from pydantic import validator
from typing import Literal

from .base_analysis import BaseAnalysisSchema
from ..._settings import settings


RSE_TYPE = Literal['SURFSARA_USERDISK',
              'SDSC_USERDISK', 
              'LNGS_USERDISK', 
              'UC_OSG_USERDISK', 
              'UC_DALI_USERDISK', 
              'CNAF_USERDISK']


def xeauth_user():
    if uconfig is None:
        return 'unknown'
    return uconfig.get('xeauth', 'api_user', fallback='unknown')


class ProcessingRequest(BaseAnalysisSchema):
    '''Schema definition for a processing request
    '''
    _ALIAS = 'processing_requests'

    data_type: str = rframe.Index()
    lineage_hash: str = rframe.Index()
    run_id: str = rframe.Index()
    destination: RSE_TYPE = rframe.Index(default='UC_DALI_USERDISK')
    user: str = pydantic.Field(default_factory=xeauth_user)
    request_date: datetime.datetime = pydantic.Field(default_factory=datetime.datetime.utcnow)
    
    priority: int = -1

    comments: str = ''
    
    
    def pre_update(self, datasource, new):
        if new.user != self.user:
            raise ValueError(new.user)
        if new.run_id != self.run_id:
            raise ValueError(new.run)


    def latest_context(self):
        import utilix
        import pymongo

        contexts = utilix.xent_collection('contexts')
        ctx = contexts.find_one({f'hashes.{self.data_type}': self.lineage_hash},
                  projection={'context': '$name', 'env': '$tag', '_id': 0},
                  sort=[('date_added', pymongo.DESCENDING)])
        
        return dict(ctx)




import os
from warnings import warn

import rframe


def api_client(url, token=None):
    headers = {}
    if token is not None:
        headers["Authorization"] = f"Bearer {token}"

    client = rframe.RestClient(
        url,
        headers=headers,
    )
    return client


def api_token(username=None, password=None, readonly=True,
             audience='https://api.cmt.xenonnt.org'):
    import xeauth
    
    if readonly:
        scopes = ['read:all', 'write:all']
    else:
        scopes = ['read:all']

    xetoken = xeauth.login(username=username, 
                           password=password, 
                           scopes=scopes, 
                           audience=audience)

    return xetoken.access_token



from typing import Literal

import rframe

from .base_corrections import TimeSampledCorrection
from ..constants import DETECTOR

class PmtGain(TimeSampledCorrection):
    _ALIAS = "pmt_gains"

    # Here we use a simple indexer (matches on exact value)
    # to define the pmt field
    # this will add the field to all documents and enable
    # selections on the pmt number. Since this is a index field
    # versioning will be indepentent for each pmt

    detector: DETECTOR = rframe.Index()
    pmt: int = rframe.Index(ge=0)

    value: float



import datetime
import time

import pytz


class SimpleClock:
    utc: bool
    cutoff_offset: float

    def __init__(self, utc=True, cutoff_offset=3600) -> None:
        self.utc = utc
        self.cutoff_offset = cutoff_offset

    def current_datetime(self):
        if self.utc:
            now = datetime.datetime.utcnow()
        else:
            now = datetime.datetime.now()
        return self.normalize_tz(now)

    def cutoff_datetime(self, buffer=0.0):
        offset = datetime.timedelta(seconds=self.cutoff_offset + buffer)
        return self.current_datetime() + offset

    def normalize_tz(self, dt: datetime.datetime) -> datetime.datetime:
        if dt.tzinfo is not None:
            if dt.tzinfo.utcoffset(dt) is not None:
                dt = dt.astimezone(pytz.utc)
            dt = dt.replace(tzinfo=None)
        dt = dt.replace(microsecond=int(dt.microsecond / 1000) * 1000)
        return dt

    def after_cutoff(self, dt, buffer=0.0):
        cutoff = self.cutoff_datetime(buffer)
        dt = self.normalize_tz(dt)
        return dt > cutoff

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v):
        if not hasattr(v, 'after_cutoff'):
            raise TypeError('Clock must implement `after_cutoff`')

        if not hasattr(v, 'cutoff_datetime'):
            raise TypeError('Clock must implement `cutoff_datetime`')

        if not hasattr(v, 'current_datetime'):
            raise TypeError('Clock must implement `current_datetime`')

        if not hasattr(v, 'normalize_tz'):
            raise TypeError('Clock must implement `normalize_tz`')

        return v



import datetime
import pydantic
import rframe
from pydantic import validator

from ..base_schemas import XeDoc
from ..._settings import settings


class BaseOperationsReport(XeDoc):
    """Base class for operations report metadata
    """
    _ALIAS = ""
    _CATEGORY = "operations"

    class Config:
        allow_population_by_field_name = True

    time: rframe.Interval[datetime.datetime] = rframe.IntervalIndex(alias='run_id')

    operator: str = pydantic.Field(min_length=1, max_length=60)
    comments: str

    @validator('time', pre=True)
    def run_id_to_time(cls, v):
        """Convert run id to time"""
        if isinstance(v, (str, int)):
            try:
                v = settings.run_id_to_interval(v)
            except:
                pass
        return v






from typing import Literal
from .base_report import BaseOperationsReport



class AnodeRampReport(BaseOperationsReport):
    """Anode ramp - report
    """

    _ALIAS = "anode_ramps"
    
    direction: Literal['up','down']
    starting_voltage: float
    final_voltage: float



"""Console script for xedocs."""
import os
import sys
import time
import xedocs
import click
import logging

from rich.logging import RichHandler
from rich.console import Console
from rich.table import Table
from rich.live import Live
from rich.progress import Progress


logging.basicConfig(
    level="NOTSET",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)]
)

logger = logging.getLogger("xedocs")

def to_str(obj):
    if isinstance(obj, (list, tuple)):
        content = ",".join([to_str(x) for x in obj])
        return f"({content})"
    if isinstance(obj, dict):
        return ",".join([f"{k}={to_str(v)}" for k, v in obj.items()])
    return f"{obj}"

@click.group()
def main():
    """Console script for xedocs."""
    click.echo("Welcome to the xedocs CLI")
    
    return 0


@main.command(name='find', context_settings=dict(
    ignore_unknown_options=True,
    allow_extra_args=True,
))
@click.argument('name')
@click.pass_context
def cli_find(ctx, name: str):
    console = Console()
    kwargs = dict([item.strip('--').split('=') for item in ctx.args])
    kwargs = {k: v.split(',') if ',' in v else v for k,v in kwargs.items()}
    try:
        schema = xedocs.find_schema(name)
    except KeyError:
        logger.error(f'Cant find a schema for `{name}`\n'
                    f'Available schemas: {xedocs.list_schemas()}'
        )
        exit(0)
    
    query_str = ", ".join([f'{k}={v}' for k,v in kwargs.items()])

    table = Table(title=f"{name.title()} query {query_str}")

    field_order = []
    for field in schema.get_index_fields():
        table.add_column(field.title(), justify="left")
        field_order.append(field)

    for field in schema.get_column_fields():
        table.add_column(field.title(), justify="center")
        field_order.append(field)

    with console.status(f"[bold green]Looking for {name} documents that match your query...") as status:
        docs = schema.find_iter(**kwargs)

    with Live(table, refresh_per_second=4):

        for doc in docs:
            data = doc.jsonable()
            row = [str(data[field]) for field in field_order]
            table.add_row(*row)
            time.sleep(0.1)


@main.command(name='download', context_settings=dict(
    ignore_unknown_options=True,
    allow_extra_args=True,
))
@click.argument('name')
@click.option('path', '--path', '-p', default='.', help='Path to download to')
@click.pass_context
def cli_download(ctx, name: str, path: str = None):
    console = Console()
    kwargs = dict([item.strip('--').split('=') for item in ctx.args])
    kwargs = {k: v.split(',') if ',' in v else v for k,v in kwargs.items()}
    try:
        schema = xedocs.find_schema(name)
    except KeyError:
        logger.error(f'Cant find a schema for `{name}`\n'
                    f'Available schemas: {xedocs.list_schemas()}'
        )
        exit(0)

    fname = f"{name}-{to_str(kwargs)}.csv"
    path = os.path.join(path, fname)
    fpath = os.path.abspath(path)

    with console.status(f"[bold green]Looking for {name} documents that match your query...") as status:
        df = schema.find_df(**kwargs)

    with console.status(f"[bold green]Saving {name} documents to {path}") as status:
        df.to_csv(fpath)
        time.sleep(1)
        
    console.print(f"Data saved to {fpath}")
   
if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tests',
 'xedocs',
 'xedocs.schemas',
 'xedocs.schemas.analysis',
 'xedocs.schemas.calibrations',
 'xedocs.schemas.corrections',
 'xedocs.schemas.operations_reports',
 'xedocs.schemas.pmt_data']

package_data = \
{'': ['*']}

install_requires = \
['click', 'rframe>=0.1.19,<0.2.0', 'rich']

extras_require = \
{'db': ['pymongo'], 'gui': ['pydantic-panel>=0.1.8,<0.2.0']}

entry_points = \
{'console_scripts': ['xedocs = xedocs.cli:main']}

setup_kwargs = {
    'name': 'xedocs',
    'version': '0.1.17',
    'description': 'Top-level package for xedocs.',
    'long_description': "===========================================\nXeDocs - AKA Corrections Managment Tool 2.0\n===========================================\nxedocs is meant to replace cmt and bodega as well as helping tracking all shared documents especially if\nthey need to be versioned.\n\nBasic Usage\n-----------\n\nExplore the available schemas\n\n.. code-block:: python\n\n    import xedocs\n\n    >>> xedocs.list_schemas()\n    >>> ['bodega',\n        'fax_configs',\n        'plugin_lineages',\n        'context_lineages',\n        'pmt_gains',\n        'global_versions',\n        'electron_drift_velocities',\n        ...]\n\n    >>> xedocs.help('pmt_gains')\n\n    >>>\n            Schema name: pmt_gains\n            Index fields: ['version', 'time', 'detector', 'pmt']\n            Column fields: ['created_date', 'comments', 'value']\n    \n\nRead data from the default data source\n\n.. code-block:: python\n\n    import xedocs\n\n    docs = xedocs.find('pmt_gains',  version='v1', pmt=[1,2,3,5], time='2021-01-01T00:00:00', detector='tpc')\n    gains = [doc.value for doc in docs]\n\n    doc = xedocs.find_one('pmt_gains',  version='v1', pmt=1, time='2021-01-01T00:00:00', detector='tpc')\n    pmt1_gain = doc.value\n\n\nYou can also query documents directly from the scham class, \nSchemas will query the mongodb cmt2 database by default, if no explicit datasource is given.\n\n.. code-block:: python\n    \n    drift_velocity = xedocs.Bodega.find_one(field='drift_velocity', version='v1')\n    \n    # Returns a Bodega object with attributes value, description etc.\n    drift_velocity.value\n\n    all_v1_documents = xedocs.Bodega.find(version='v1')\n\n\n\nRead data from alternative data sources specified by path, \ne.g csv files which will be loaded by pandas.\n\n.. code-block:: python\n\n    import xedocs\n    \n    g1_doc = xedocs.find_one('bodega', datasource='/path/to/file.csv', version='v1', field='g1')\n    g1_value = g1_doc.value\n    g1_error = g1_doc.uncertainty\n\nThe path can also be a github URL or any other URL supported by fsspec. \n\n.. code-block:: python\n\n    import xedocs\n    \n    g1_doc = xedocs.find_one('bodega',\n                             datasource='github://org:repo@/path/to/file.csv', \n                             version='v1', \n                             field='g1')\n\n\nSupported data sources\n\n    - MongoDB collections\n    - TinyDB tables\n    - JSON files\n    - REST API clients\n\nPlease open an issue on rframe_ if you want support for an additional data format.\n\n\nDocumentation\n-------------\nFull documentation hosted by Readthedocs_\n\nCredits\n-------\n\n\nThis package was created with Cookiecutter_ and the `briggySmalls/cookiecutter-pypackage`_ project template.\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`briggySmalls/cookiecutter-pypackage`: https://github.com/briggySmalls/cookiecutter-pypackage\n.. _Readthedocs: https://xedocs.readthedocs.io/en/latest/\n.. _rframe: https://github.com/jmosbacher/rframe",
    'author': 'Yossi Mosbacher',
    'author_email': 'joe.mosbacher@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/XENONnT/xedocs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



"""
# S2(x,y) correction codes
**Jianyu Long (jylong@uchicago.edu)**

Time dependence of Extraction Efficiency generated from Kr83m calibration data and propagated to all runs. As discussed in [this note](https://xe1t-wiki.lngs.infn.it/doku.php?id=jlong:sr0_ramp_up_kr_se_study#updated_correction_maps) the extraction efficiency seems have a strong correlation with time delay from the nearest ramp up. This dependence should be regarded together with Single Electron Gain evolution (also in that note). 

## Brief info

- This correction is generated by interpolating EE as a function of time delay from the ramp up for Kr83m calibration runs during 5.3.2021 and 5.15.2021
- V0 is just for qualitative purpose and Straxen algorithm test
- The json file contains two meaningful fields: `timestamps` and `correction`: `timestamps` are simply starting time for all available runs; `correction` are the associated correction coefficients. 

"""

from .base_corrections import TimeSampledCorrection


class RelExtractionEff(TimeSampledCorrection):
    _ALIAS = "rel_extraction_effs"

    value: float



"""Top-level package for xedocs."""

__author__ = """Yossi Mosbacher"""
__email__ = "joe.mosbacher@gmail.com"
__version__ = "0.1.17"
import logging

logger = logging.getLogger(__name__)

from ast import Import
from ._settings import settings
from ._frames import frames
from . import schemas
from .schemas import *
from .utils import *
from .xedocs import *
from . import api

try:
    from . import widgets
    gui = widgets.XedocsEditor()
except ImportError:
    logger.warning('Could not import editors, GUI not available.')


"""
# S1 AFT (x,y,z) map
S1 area fraction top data driven map used for event_patternfit within straxen. The map is data-driven and depends on the PMT Gains and other observables. Should be treated like a normal correction for dependencies and should be updated last for each global version.

"""
